<?php

namespace YPS\WC_Conditional_Cart_Notices;

use YPS\WC_Conditional_Cart_Notices\Framework\Core\Controller;
use YPS\WC_Conditional_Cart_Notices\Framework\Core\Helper;

class Notice_Helper extends Helper {

    public function __construct($context, $params = array()) {
        parent::__construct($context, $params);

        /* CART */
        add_action('woocommerce_before_cart', array($this, 'woocommerce_before_cart'), 10);
        add_action('woocommerce_before_cart_table', array($this, 'woocommerce_before_cart_table'), 10);
        add_action('woocommerce_before_cart_contents', array($this, 'woocommerce_before_cart_contents'), 10);
        
        add_action('woocommerce_cart_contents', array($this, 'woocommerce_cart_contents'), 10);
        add_action('woocommerce_after_cart_contents', array($this, 'woocommerce_after_cart_contents'), 10);
        add_action('woocommerce_after_cart_table', array($this, 'woocommerce_after_cart_table'), 10);

        /* CHECKOUT */
        add_action('woocommerce_before_checkout_form', array($this, 'woocommerce_before_checkout_form'), 10);
        add_action('woocommerce_checkout_before_customer_details', array($this, 'woocommerce_checkout_before_customer_details'), 10);
        add_action('woocommerce_before_checkout_billing_form', array($this, 'woocommerce_before_checkout_billing_form'), 10);
        add_action('woocommerce_after_checkout_billing_form', array($this, 'woocommerce_after_checkout_billing_form'), 10);
        add_action('woocommerce_before_checkout_shipping_form', array($this, 'woocommerce_before_checkout_shipping_form'), 10);
        add_action('woocommerce_after_checkout_shipping_form', array($this, 'woocommerce_after_checkout_shipping_form'), 10);

        /* PRODUCT */
        add_action('woocommerce_before_single_product', array($this, 'woocommerce_before_single_product'), 10);
        add_action('woocommerce_before_product_summary', array($this, 'woocommerce_before_product_summary'), 10);
        add_action('woocommerce_single_product_summary', array($this, 'woocommerce_single_product_summary'), 10);
        add_action('woocommerce_before_variations_form', array($this, 'woocommerce_before_variations_form'), 10);
        add_action('woocommerce_after_variations_form', array($this, 'woocommerce_after_variations_form'), 10);
        add_action('woocommerce_after_single_product_summary', array($this, 'woocommerce_after_single_product_summary'), 10);
        
        
        /* SHOP */
        add_action('woocommerce_before_shop_loop', array($this, 'woocommerce_before_shop_loop'), 10);
        add_action('woocommerce_after_shop_loop', array($this, 'woocommerce_after_shop_loop'), 10);

        if(apply_filters('yps_wc_conditional_cart_notices_is_pro', false) == true){
            add_shortcode("yps_wc_conditional_cart_notices", array($this, 'shortcode_yps_wc_conditional_cart_notices'));
        }
        
    }

    public function shortcode_yps_wc_conditional_cart_notices(){
        @ob_start();
        $this->process_notices();
        $contents       = @ob_get_contents();
        @ob_end_clean();

        return $contents;
    }

    public function woocommerce_before_shop_loop(){
        $this->process_notices('shop_position', Notice_Form::SHOP_POSITION_BEFORE_SHOP_LOOP);
    }

    public function woocommerce_after_shop_loop(){
        $this->process_notices('shop_position', Notice_Form::SHOP_POSITION_AFTER_SHOP_LOOP);
    }

    public function woocommerce_before_single_product(){
        $this->process_notices('product_position', Notice_Form::PRODUCT_POSITION_BEFORE_SINGLE_PRODUCT);
    }

    public function woocommerce_before_product_summary(){
        $this->process_notices('product_position', Notice_Form::PRODUCT_POSITION_BEFORE_PRODUCT_SUMMARY);
    }

    public function woocommerce_single_product_summary(){
        $this->process_notices('product_position', Notice_Form::PRODUCT_POSITION_SINGLE_PRODUCT_SUMMARY);
    }

    public function woocommerce_before_variations_form(){
        $this->process_notices('product_position', Notice_Form::PRODUCT_POSITION_BEFORE_VARIATIONS_FORM);
    }

    public function woocommerce_after_variations_form(){
        $this->process_notices('product_position', Notice_Form::PRODUCT_POSITION_AFTER_VARIATIONS_FORM);
    }

    public function woocommerce_after_single_product_summary(){
        $this->process_notices('product_position', Notice_Form::PRODUCT_POSITION_AFTER_SINGLE_SUMMARY);
    }

    public function woocommerce_after_checkout_shipping_form(){
        $this->process_notices('checkout_position', Notice_Form::CHECKOUT_POSITION_AFTER_SHIPPING_FORM);
    }

    public function woocommerce_before_checkout_shipping_form(){
        $this->process_notices('checkout_position', Notice_Form::CHECKOUT_POSITION_BEFORE_SHIPPING_FORM);
    }

    public function woocommerce_after_checkout_billing_form(){
        $this->process_notices('checkout_position', Notice_Form::CHECKOUT_POSITION_AFTER_BILLING_FORM);
    }

    public function woocommerce_before_checkout_billing_form(){
        $this->process_notices('checkout_position', Notice_Form::CHECKOUT_POSITION_BEFORE_BILLING_FORM);
    }

    public function woocommerce_checkout_before_customer_details(){
        $this->process_notices('checkout_position', Notice_Form::CHECKOUT_POSITION_BEFORE_CUSTOMER_DETAILS);
    }

    public function woocommerce_before_checkout_form(){
        $this->process_notices('checkout_position', Notice_Form::CHECKOUT_POSITION_BEFORE_FORM);
    }

    public function woocommerce_before_cart(){
        $this->process_notices('cart_position', Notice_Form::CART_POSITION_BEFORE_CART);
    }

    public function woocommerce_before_cart_table(){
        $this->process_notices('cart_position', Notice_Form::CART_POSITION_AFTER_CART_TABLE);
    }

    public function woocommerce_before_cart_contents(){
        $this->process_notices('cart_position', Notice_Form::CART_POSITION_BEFORE_CART_CONTENTS);
    }

    public function woocommerce_cart_contents(){
        $this->process_notices('cart_position', Notice_Form::CART_POSITION_AFTER_CART);
    }

    public function woocommerce_after_cart_contents(){
        $this->process_notices('cart_position', Notice_Form::CART_POSITION_AFTER_CART_CONTENTS);
    }

    public function woocommerce_after_cart_table(){
        $this->process_notices('cart_position', Notice_Form::CART_POSITION_AFTER_CART_TABLE);
    }

    public static function is_notice_visible($notice_entity, $position_field = null, $position_value = null){
        if(
            is_cart() == true && in_array(Notice_Form::SHOW_NOTICE_IN_CART, $notice_entity->get('show_notice_in')) == true ||
            apply_filters('yps_wc_conditional_cart_notices_show_notice', false, $notice_entity) == true ||
            empty($position_field) && empty($position_value)
        ){
            return true;
        }

        return false;
    }

    public function process_notices($position_field = null, $position_value = null){

        $notice_model               = new Notice_Model($this->context);
        $settings_model             = new Settings_Model($this->context);
        $controller                 = new Controller($this->context, $this->params);

        if(is_cart() == true && $settings_model->get_value("show_in_cart") == false){
            return;
        }

        if(is_checkout() == true && $settings_model->get_value("show_in_checkout") == false){
            return;
        }

        if(is_product() == true && $settings_model->get_value("show_in_product") == false){
            return;
        }

        if(is_shop() == true && $settings_model->get_value("show_in_shop") == false){
            return;
        }

        $where                      = array(
            'enabled'       => true
        );

        if(!empty($position_field) && !empty($position_value)){
            $where[$position_field]     = $position_value;
        }

        $notices                = $notice_model->get_entities_by_columns($where);

        echo $controller->get_view("notice", "notices.php", array(
            'controller'        => $controller,
            'notices'           => $notices,
            'position_field'    => $position_field,
            'position_value'    => $position_value
        ));

    }
        
}
