<?php

namespace YPS\WC_Conditional_Cart_Notices;

use YPS\WC_Conditional_Cart_Notices\Framework\Core\Helper;

class Query_Builder_Helper extends Helper {

    public function create_tmp_table_from_cart($notice_entity){

        $query_builder_model    = new Query_Builder_Model($this->context);

        if(is_user_logged_in()){
            $user_types             = "logged-in-users";
            $user                   = wp_get_current_user();
            $roles                  = (array)$user->roles;

            $billing_country        = get_user_meta($user->ID, 'billing_country', true);
            $shipping_country       = get_user_meta($user->ID, 'shipping_country', true);
        }else{
            $user_types             = "not-logged-in-users";
            $roles                  = array('visitor');

            $billing_country        = "";
            $shipping_country       = "";
        }
        
        $query_builder_model->create_table();

        /* Totals */

        $total_quantity                 = 0;
        $total_amount                   = 0;
        $total_net_amount               = 0;
        $total_taxes                    = 0;

        $total_coupon_discount_amount   = \WC()->cart->get_cart_discount_total();
        $total_coupon_tax_total         = WC()->cart->get_cart_discount_tax_total();

        $shipping_total                 = \WC()->cart->get_shipping_total();

        foreach(\WC()->cart->get_tax_totals() as $tax){
            $total_taxes        += $tax->amount;
        }

        foreach(\WC()->cart->get_cart() as $cart_item){
            $total_quantity     += $cart_item['quantity'];
            $total_amount       += $cart_item['quantity']*\wc_get_price_including_tax($cart_item['data']);
            $total_net_amount   += $cart_item['quantity']*\wc_get_price_excluding_tax($cart_item['data']);
        }

        if($notice_entity->get('include_coupons_in_total_amount') == true){
            $total_amount       += $total_coupon_discount_amount;
        }

        if($notice_entity->get('include_coupons_taxes_in_total_amount') == true){
            $total_amount       += $total_coupon_tax_total;
        }

        if($notice_entity->get('include_taxes_in_total_amount') == true){
            $total_amount       -= $total_taxes;
        }

        if($notice_entity->get('include_shipping_taxes_in_total_amount') == true){
            $total_amount       -= $shipping_total;
        }

        /* Single */
        foreach(\WC()->cart->get_cart() as $cart_item){
            $product_id     = $cart_item['data']->get_id();
            $item_name      = $cart_item['data']->get_title();
            $quantity       = $cart_item['quantity'];
            $price          = $cart_item['data']->get_price();
            $categories     = \get_the_terms($product_id, 'product_cat');

            foreach($categories as $category){
                foreach($roles as $role){
                    $query_builder_model->insert_row(apply_filters('yps_wc_conditional_cart_notices_query_builder_row', array(
                        'categories'            => $category->term_id,
                        'products'              => $product_id,
                        'user_roles'            => $role,
                        'user_types'            => $user_types,

                        'billing_country'       => $billing_country,
                        'shipping_country'      => $shipping_country,

                        'datetime'              => current_time('mysql'),

                        'price'                 => $price,

                        'product_quantity'      => $quantity,

                        'total_quantity'                => $total_quantity,
                        'total_amount'                  => $total_amount,
                        'total_net_amount'              => $total_net_amount,
                        'total_taxes'                   => $total_taxes,
                        'shipping_total'                => $shipping_total,
                        'total_coupon_discount_amount'  => $total_coupon_discount_amount,

                        'referrer_url'                  => (isset($_SERVER['HTTP_REFERER']))?$_SERVER['HTTP_REFERER']:"",
                    )));
                }
            }

        }

    }

    public function drop_tmp_table(){
        $query_builder_model    = new Query_Builder_Model($this->context);

        $query_builder_model->query("DROP TEMPORARY TABLE IF EXISTS {$query_builder_model->table}");
    }

        
}
