<?php

namespace YPS\WC_Conditional_Cart_Notices;

use YPS\WC_Conditional_Cart_Notices\Framework\Form\Form;
use YPS\WC_Conditional_Cart_Notices\Framework\Form\Color_Form_Field;
use YPS\WC_Conditional_Cart_Notices\Framework\Form\Form_Field;
use YPS\WC_Conditional_Cart_Notices\Framework\Form\Form_Group;
use YPS\WC_Conditional_Cart_Notices\Framework\Form\WP_Media_Form_Field;

use YPS\WC_Conditional_Cart_Notices\Framework\Database\Database_Helper;


class Template_Form extends Form {
    
    public function __construct($context, $params = array()) {
        parent::__construct($context, $params);

        $basic_group            = new Form_Group($this->context);
        $box_group              = new Form_Group($this->context);
        $icon_group             = new Form_Group($this->context);
        $button_group           = new Form_Group($this->context);

        $basic_group
            ->set_name('basic')
            ->set_label(__('Template Basic Settings', 'yps-wc-conditional-cart-notices'))
            ->set_wrapper_classes('mb-5');

        $box_group
            ->set_name('box')
            ->set_label(__('Box Settings', 'yps-wc-conditional-cart-notices'))
            ->set_wrapper_classes('mb-5');

        $icon_group
            ->set_name('icon')
            ->set_label(__('Icon Settings', 'yps-wc-conditional-cart-notices'))
            ->set_wrapper_classes('mb-5');

        $button_group
            ->set_name('button')
            ->set_label(__('Button Settings', 'yps-wc-conditional-cart-notices'))
            ->set_wrapper_classes('mb-5');

        $this->add_group($basic_group);
        $this->add_group($box_group);
        $this->add_group($icon_group);
        $this->add_group($button_group);

        $id                                 = new Form_Field($context, $params);
        $name                               = new Form_Field($context, $params);

        $box_background_color               = new Color_Form_Field($context, $params);
        $box_border_color                   = new Color_Form_Field($context, $params);
        $box_text_color                     = new Color_Form_Field($context, $params);

        $button_background_color            = new Color_Form_Field($context, $params);
        $button_border_color                = new Color_Form_Field($context, $params);
        $button_background_color_hover      = new Color_Form_Field($context, $params);
        $button_border_color_hover          = new Color_Form_Field($context, $params);
        $button_text_color                  = new Color_Form_Field($context, $params);

        $icon_image                         = new WP_Media_Form_Field($context, $params);
        $icon_background_color              = new Color_Form_Field($context, $params);

        $id
            ->set_name('id')
            ->set_label(__('ID', 'yps-wc-conditional-cart-notices'))
            ->set_type(Form_Field::TYPE_TEXT)
            ->set_allow_empty(true)
            ->set_group_name('basic')
            ->set_group_row_number(0)
            ->set_is_primary_key(true)
            ->set_sql_column(Database_Helper::PRIMARY_KEY_UINT)
            ->set_wrapper_classes('col-lg-3 col-sm-6')
            ->set_is_table_header(true)
            ->set_is_table_edit_url(true)
            ->set_attributes(array('class' => array('form-control'), 'readonly' => 'readonly'));

        $name
            ->set_name('name')
            ->set_label(__('Name', 'yps-wc-conditional-cart-notices'))
            ->set_type(Form_Field::TYPE_TEXT)
            ->set_allow_empty(false)
            ->set_group_name('basic')
            ->set_group_row_number(0)
            ->set_sql_column('MEDIUMTEXT DEFAULT NULL')
            ->set_wrapper_classes('col-lg-3 col-sm-6')
            ->set_is_table_header(true)
            ->set_is_table_edit_url(true)
            ->set_attributes(array('class' => array('form-control')));

        $box_background_color
            ->set_name('box_background_color')
            ->set_label(__('Background Color', 'yps-wc-conditional-cart-notices'))
            ->set_allow_empty(true)
            ->set_group_name('box')
            ->set_group_row_number(0)
            ->set_sql_column('VARCHAR(255) DEFAULT NULL')
            ->set_wrapper_classes('col-lg-4 col-sm-12');

        $box_border_color
            ->set_name('box_border_color')
            ->set_label(__('Border Color', 'yps-wc-conditional-cart-notices'))
            ->set_allow_empty(true)
            ->set_group_name('box')
            ->set_group_row_number(0)
            ->set_sql_column('VARCHAR(255) DEFAULT NULL')
            ->set_wrapper_classes('col-lg-4 col-sm-12');

        $box_text_color
            ->set_name('box_text_color')
            ->set_label(__('Text Color', 'yps-wc-conditional-cart-notices'))
            ->set_allow_empty(true)
            ->set_group_name('box')
            ->set_group_row_number(0)
            ->set_sql_column('VARCHAR(255) DEFAULT NULL')
            ->set_wrapper_classes('col-lg-4 col-sm-12');

        $button_background_color
            ->set_name('button_background_color')
            ->set_label(__('Background Color', 'yps-wc-conditional-cart-notices'))
            ->set_allow_empty(true)
            ->set_group_name('button')
            ->set_group_row_number(0)
            ->set_sql_column('VARCHAR(255) DEFAULT NULL')
            ->set_wrapper_classes('col-lg-6 col-sm-12');

        $button_border_color
            ->set_name('button_border_color')
            ->set_label(__('Border Color', 'yps-wc-conditional-cart-notices'))
            ->set_allow_empty(true)
            ->set_group_name('button')
            ->set_group_row_number(0)
            ->set_sql_column('VARCHAR(255) DEFAULT NULL')
            ->set_wrapper_classes('col-lg-6 col-sm-12');

        $button_background_color_hover
            ->set_name('button_background_color_hover')
            ->set_label(__('Background Color on hover', 'yps-wc-conditional-cart-notices'))
            ->set_allow_empty(true)
            ->set_group_name('button')
            ->set_group_row_number(1)
            ->set_sql_column('VARCHAR(255) DEFAULT NULL')
            ->set_wrapper_classes('col-lg-6 col-sm-12');

        $button_border_color_hover
            ->set_name('button_border_color_hover')
            ->set_label(__('Border Color on hover', 'yps-wc-conditional-cart-notices'))
            ->set_allow_empty(true)
            ->set_group_name('button')
            ->set_group_row_number(1)
            ->set_sql_column('VARCHAR(255) DEFAULT NULL')
            ->set_wrapper_classes('col-lg-6 col-sm-12');

        $button_text_color
            ->set_name('button_text_color')
            ->set_label(__('Text Color', 'yps-wc-conditional-cart-notices'))
            ->set_allow_empty(true)
            ->set_group_name('button')
            ->set_group_row_number(2)
            ->set_sql_column('VARCHAR(255) DEFAULT NULL')
            ->set_wrapper_classes('col-lg-6 col-sm-12');

        $icon_image
            ->set_name('icon_image')
            ->set_label(__('Icon Image', 'yps-wc-conditional-cart-notices'))
            ->set_allow_empty(true)
            ->set_group_name('icon')
            ->set_group_row_number(0)
            ->set_sql_column('MEDIUMTEXT DEFAULT NULL')
            ->set_wrapper_classes('col-lg-6 col-sm-12');

        $icon_background_color
            ->set_name('icon_background_color')
            ->set_label(__('Background Color', 'yps-wc-conditional-cart-notices'))
            ->set_allow_empty(true)
            ->set_group_name('icon')
            ->set_group_row_number(0)
            ->set_sql_column('VARCHAR(255) DEFAULT NULL')
            ->set_wrapper_classes('col-lg-6 col-sm-12');

        $this->add_field($id);
        $this->add_field($name);

        $this->add_field($box_background_color);
        $this->add_field($box_border_color);
        $this->add_field($box_text_color);

        $this->add_field($button_background_color);
        $this->add_field($button_border_color);
        $this->add_field($button_background_color_hover);
        $this->add_field($button_border_color_hover);
        $this->add_field($button_text_color);

        $this->add_field($icon_image);
        $this->add_field($icon_background_color);

        
    }
    
    public function validate($data, $params = array()){
        
        $validate               = parent::validate($data, $params);
        
        return $validate;
    }
}

