<?php

namespace YPS\Framework\Record\v481_380_927;

use YPS\Framework\Core\v481_380_927\Controller;
use YPS\Framework\Core\v481_380_927\Helper;
use YPS\Framework\Core\v481_380_927\View;
use YPS\Framework\Database\v481_380_927\Search_Query;

use YPS\Framework\Wordpress\v481_380_927\Wordpress_Helper;

class Record_Controller extends Controller {

    public $show_actions_new_button    = true;
    
    public $record_singular_name;
    public $record_plural_name;
    
    public $record_config;
    public $record_table;
    public $record_model;
    
    public $record_form;
    
    public $record_id;
    public $record_entity;
    
    public $message;
    
    public $list_title;
    public $list_show_card              = true;

    protected $edit_url                 = null;

    protected $clone                    = false;
    protected $raw                      = false;

    protected $form_view                = null;

    public function __construct($context, $params = array()) {        
        parent::__construct($context, $params);
    }
    
    public function custom_load(){

        if(Helper::get_request("raw") == true){
            $this->record_id             = (isset($_REQUEST['form_data']))?intval($_REQUEST['form_data']['id']):null;
        }else{
            $this->record_id             = Helper::get_request("id");
        }
        
        $this->message               = sprintf(__('"%s" has been saved correctly!', 'yps-framework-core'), $this->record_singular_name);
        
        if(!empty($this->record_id)){
            $this->record_entity              = $this->get_custom_entity($this->record_model->get_entity($this->record_id));
        }else{
            $this->record_entity              = $this->get_custom_entity(new $this->record_entity($this->context, $this->params));
        }

        $this->form_view                       = new Record_View($this->context, $this->params);

        $this->clone                           = Helper::get_request("clone");
        $this->raw                             = Helper::get_request("raw");
    }

    public function index_action(){
        echo $this->get_framework_view('Record', 'record/list.php', array(
            'list_title'        => $this->list_title,
            'list_show_card'    => $this->list_show_card,
            'custom_buttons'    => $this->get_index_custom_buttons(),
        ));
    }
    
    public function get_list_after_buttons_view(){
        return null;
    }

    public function get_list_header_view(){
        return $this->get_framework_view('Record', 'record/list-header.php');
    }



    public function get_list_footer_view(){

    }
    
    public function get_list_title(){
        return $this->record_plural_name;
    }

    public function get_list_description(){
        return "";
    }
    
    public function get_index_custom_buttons(){
        return "";
    }
        
    public function get_edit_view_json_data($json_data){
        return $json_data;
    }

    public function get_edit_view_params($params){
        return $params;
    }
    
    public function get_custom_entity($entity){
        return $entity;
    }

    public function edit_action(){
        
        $validate       = array(
            'errors'        => array(),
            'warnings'      => array()
        );
        
        $message        = null;
    
        if(Helper::get_request("form_task") == "save" || Helper::get_request("ajax_action") == "submit"){

            $data               = $this->record_form->get_request_data();
            $data['id']         = $this->record_id;
            $data               = $this->get_custom_action_data(__FUNCTION__, $data);

            $data               = $this->custom_change_data_on_save($data);

            Wordpress_Helper::write_log($data);

            $validate           = $this->record_form->validate($data, array(
                'record_id'                 => $this->record_id,
                'record_entity'             => $this->record_entity,
            ));
            
            $this->record_entity->set_form_data($this->record_form, $data);

            if(count($validate['errors']) == 0){
                $message            = $this->message;
                $this->record_id    = $this->record_model->save($this->record_entity);
                
                $this->record_entity->set('id', $this->record_id);

                $this->custom_after_edit_save();

                if($this->raw == false){
                    $this->redirect($this->record_config->get_edit_url($this->record_id, $this->raw));
                }
                
            }
        }else if(Helper::get_request("ajax_action") == "reload"){
            $data               = $this->record_form->get_request_data();
            $data['id']         = $this->record_id;

            $this->record_entity->set_form_data($this->record_form, $data);
        }
        
        if($this->clone == true){
            $this->record_entity->set('id', null);
        }

        

        $this->form_view->set_framework_template("Form", "form/form.php");

        $this->form_view->set_view_params($this->get_edit_view_params(array(
            'form'                      => $this->record_form,
            'entity'					=> $this->record_entity,

            'ajax_url'                  => $this->record_config->get_edit_url($this->record_id, true),
            'message'                   => $message,
            'form_validate'             => $validate,
        )));

        $this->form_view->set_view_json_data($this->get_edit_view_json_data(array(
            'status'    => (count($validate['errors']) == 0)?true:false,
            'type'      => 'ajax-form',
            'record_id' => $this->record_id
        )));

        if($this->raw == false){
            $this->form_view->set_output_mode(View::VIEW_OUTPUT_MODE_HTML);
        }else{
            $this->form_view->set_output_mode(View::VIEW_OUTPUT_MODE_HTML_AND_JSON);
        }

        return $this->form_view;
    }

    public function delete_action(){
        $this->record_model->delete_row($this->record_id);
        
        $this->custom_after_delete();

        if($this->raw == true){

            /* Returns JSON response */
            $this->set_response(array(
                'status'    => true,
                'record_id' => $this->record_id
            ), Controller::RESPONSE_JSON);

        }else{
            /* Redirect via HTML */
            $this->redirect($this->record_config->get_list_url());
        }
        
    }
    
    /**
     * @deprecated Use "custom_after_delete" instead
     */
    public function after_delete(){
        throw new \Exception("Use 'custom_after_delete' instead");
        
    }
    
    /**
     * @deprecated Use "custom_after_delete" instead
     */
    public function edit_after_save(){
        throw new \Exception("Use 'custom_after_edit_save' instead");
    }
    
    public function get_custom_action_data($action, $data){
        return $data;
    }
    
    public function list_ajax_action(){
        
        $search_query   = $this->get_list_ajax_search_query();
        
        $start          = Helper::get_request("start");
        $length         = Helper::get_request("length");
        $search_value   = Helper::get_request("search");
        $order          = Helper::get_request("order");
        
        $header         = $this->record_table->get_header();
        $header_keys    = array_keys($header);
        
        if(!empty($order)){
            $order_column   = $header_keys[$order[0]['column']];
            $order_dir      = $order[0]['dir'];
        }

        $search_query->set_search_conditions($this->get_list_ajax_search_conditions($search_value));
        
        if(empty($length)){
            $length     = 10;
        }
        
        $search_query->set_start($start);
        $search_query->set_limit($length);
        $search_query->set_order_column($order_column);
        $search_query->set_order_dir($order_dir);

        $rows           = $this->record_model->search($search_query, true, $records_total);
        
        die(json_encode($this->record_table->get_table_data($rows, $records_total)));
    }
    
    public function set_edit_message($message){
        $this->message      = $message;
    }
    
    public function get_list_ajax_search_query(){
        return new Search_Query($this->context, $this->params);
    }
    
    public function get_list_ajax_search_conditions($search_value){
        return array();
    }
    
    public function set_record_entity($record_entity){
        $this->record_entity        = $record_entity;
    }
    
    public function set_record_config($record_config){
        $this->record_config        = $record_config;
    }
    
    public function set_record_table($record_table){
        $this->record_table         = $record_table;
    }
    
    public function set_record_model($record_model){
        $this->record_model         = $record_model;
    }
    
    public function set_record_form($record_form){
        $this->record_form          = $record_form;
    }
    
    public function set_list_title($list_title){
        $this->list_title           = $list_title;
    }
    
    public function set_record_singular_name($record_singular_name){
        $this->record_singular_name     = $record_singular_name;
    }
    
    public function set_record_plural_name($record_plural_name){
        $this->record_plural_name     = $record_plural_name;
    }

    /**
     * Executed after the 'delete_action'
     */
    public function custom_after_delete(){}

    /**
     * Executed after the record has been saved in edit view (edit_action)
     */
    public function custom_after_edit_save(){}
    
    /**
     * Change requested data on save
     */
    public function custom_change_data_on_save($data){
        return $data;
    }

    /**
     * Get the value of edit_url
     */ 
    public function get_edit_url()
    {
        if($this->edit_url === null){
            return $this->record_config->get_edit_url();
        }
        
        return $this->edit_url;
    }

    /**
     * Set the value of edit_url
     *
     * @return  self
     */ 
    public function set_edit_url($edit_url)
    {
        $this->edit_url = $edit_url;

        return $this;
    }

    /**
     * Get the value of clone
     */ 
    public function get_is_clone()
    {
        return $this->clone;
    }

    /**
     * Set the value of clone
     *
     * @return  self
     */ 
    public function set_is_clone($is_clone)
    {
        $this->clone = $is_clone;

        return $this;
    }
}
